precision highp float;

__USER_DEFINES__

#define TEX2D(tex, pos) texture2D(tex, pos)

#my_include "./projections.inc.glsl"

//======================================================================
//Skeleton for ventusky methods with 1 parametr
//only one input texture is used
// (basically all except wind)
//======================================================================


varying vec2 vTexCoord; //pro projekce


uniform vec4 minMaxStepOpacity;

uniform sampler2D en_texModel;

//#ifdef SMOOTH_TRANSITION
uniform sampler2D en_texModelPrev;
uniform float smoothStep;
//#endif

uniform sampler2D colorMap;
uniform vec2 colorMapSize;

uniform float enableInterpolation;

uniform vec4 texCoordAtlas;

float ConvertPackedValue(__CONVERT_PACKED_VALUE_PARAM__) __CONVERT_PACKED_VALUE_FUNCTION__


vec4 GetUnpackedColor(float val)
{	
	float intervalLength = (minMaxStepOpacity.y - minMaxStepOpacity.x);
	float tx = (val - minMaxStepOpacity.x) / intervalLength;
    
	//move to a center of texel
	//without this, small values were cutoff from color map
	//because in app values are rounded 
	//(eg. (0 - 0.4) to 0 and (0.5 - 0.9) to 1. Without this, (0 - 0.9) was 0)
	tx += 0.5 * colorMapSize.y;

	return texture2D(colorMap, vec2(tx, 0.5));
}

#my_include "ventusky/ventusky_layer_interpolations.ps.glsl"



float GetUnpackedValue(vec2 texCoord, sampler2D tex)
{
	float unpackedValue = 0.0;

	if (enableInterpolation < 0.5)
	{
		float packedValue = TEX2D(tex, texCoord).r;
		unpackedValue = ConvertPackedValue(packedValue * 255.0);
	}
	else
	{
#ifdef NON_LINEAR
		//for non-linear convert function, interpolation 
		//cannot be done on "texture" values, but must be done
		//on "unpacked" values
		//in this case, we must run interpolation manually

	#ifdef USE_BICUBIC
		#ifdef USE_LOW_QUALITY			
			unpackedValue = UnpackValue_BilinearInterpolation_NonLinearTexValues(texCoord, tex);
		#else
			unpackedValue = UnpackValue_BicubicInterpolation_NonLinearTexValues(texCoord, tex);
		#endif
	#else
		unpackedValue = UnpackValue_BilinearInterpolation_NonLinearTexValues(texCoord, tex);
#endif


#else
	#ifdef USE_BICUBIC
		#ifdef USE_LOW_QUALITY
			float packedValue = TEX2D(tex, texCoord).r;
		#else
			float packedValue = UnpackValue_BicubicInterpolation_LinearTexValues(texCoord, tex);
		#endif
	#else
		float packedValue = TEX2D(tex, texCoord).r;
	#endif
		unpackedValue = ConvertPackedValue(packedValue * 255.0);
#endif
	}

	return unpackedValue;
}

void main()
{		
	vec2 texCoord = vTexCoord + texCoordAtlas.xy;
	
	float unpackedValue = GetUnpackedValue(texCoord, en_texModel);

	if (smoothStep < 1.0)
	{		
		float unpackedValuePrev = GetUnpackedValue(texCoord, en_texModelPrev);
		unpackedValue = mix(unpackedValuePrev, unpackedValue, smoothStep);	
	}
    
	vec4 color = GetUnpackedColor(unpackedValue);	
	gl_FragColor = color;
	gl_FragColor.a *= minMaxStepOpacity.w;	//*= multiply opacity from "color map" with "user defined"

	
	//DEBUG	
    //if ((vTexCoord.x < 0.005) || (vTexCoord.x > 0.995)) gl_FragColor.rgb = vec3(1,1,0);
    //if ((vTexCoord.y < 0.005) || (vTexCoord.y > 0.995)) gl_FragColor.rgb = vec3(1,1,0);
    
}
