precision highp float;

#define TEX2D(tex, pos) texture2D(tex, pos)


varying vec2 vTexCoord; //pro projekce
varying vec2 vWorldPos; //pro projekce
varying vec2 gps;

uniform sampler2D en_texModel;

uniform float typeMapScale;
uniform vec4 opacityRainStormFreezeSnow;
uniform float opacityFog;

uniform sampler2D snowMap;
uniform sampler2D stormMap;
uniform sampler2D rainMap;
uniform sampler2D fogMap;
uniform sampler2D freezeMap;

float ConvertPackedValue(float b) 
{   
	return b;
}

#my_include "./projections.inc.glsl"
#my_include "./ventusky/ventusky_layer_interpolations.ps.glsl"




bool IsRain_NoInterpolation(float unpackedVal)
{
	float tmpRain = abs(unpackedVal - 10.0);
	return (tmpRain < 1.0);
}

bool IsSnow_NoInterpolation(float unpackedVal)
{
	float tmpSnow = abs(unpackedVal - 30.0);
	return (tmpSnow < 1.0);
}

bool IsStorm_NoInterpolation(float unpackedVal)
{
	float tmpStorm = abs(unpackedVal - 40.0);
	return (tmpStorm < 1.0);
}

bool IsFreezeRain_NoInterpolation(float unpackedVal)
{
	float tmpFreezeRain = abs(unpackedVal - 50.0);
	return (tmpFreezeRain < 1.0);
}

bool IsFog_NoInterpolation(float unpackedVal)
{
	float tmpFog = abs(unpackedVal - 70.0);
	return (tmpFog < 1.0);
}

//================================================================================

float BilinearInterpolationForType(vec4 unpackedValue, vec2 f, float type)
{
	vec4 tmp = abs(unpackedValue - vec4(type));
	unpackedValue = vec4(lessThan(tmp, vec4(1.0))) * vec4(type);

	float tA = mix(unpackedValue.x, unpackedValue.y, f.x);
	float tB = mix(unpackedValue.z, unpackedValue.w, f.x);
	return mix(tA, tB, f.y);
}

bool IsRain_BilinearInterpolation(vec4 unpackedValue, vec2 f)
{
	return IsRain_NoInterpolation(BilinearInterpolationForType(unpackedValue, f, 10.0));
}

bool IsStorm_BilinearInterpolation(vec4 unpackedValue, vec2 f)
{
	return IsStorm_NoInterpolation(BilinearInterpolationForType(unpackedValue, f, 40.0));
}

bool IsFreezeRain_BilinearInterpolation(vec4 unpackedValue, vec2 f)
{
	return IsFreezeRain_NoInterpolation(BilinearInterpolationForType(unpackedValue, f, 50.0));
}

bool IsSnow_BilinearInterpolation(vec4 unpackedValue, vec2 f)
{
	return IsSnow_NoInterpolation(BilinearInterpolationForType(unpackedValue, f, 30.0));
}



float PrepareNoInterpolationForTypes(vec2 iTc, sampler2D tex)
{
#ifdef GLES_2	
	return texture2D(tex, iTc).a * 255.0;
#else	
	return texture2D(tex, iTc).r * 255.0;
#endif
}

vec4 PrepareBilinearForTypes(vec2 iTc, sampler2D tex, out vec2 f)
{
	vec2 tc0 = vec2(0.0);
	vec2 tc1 = vec2(0.0);
	vec2 tc2 = vec2(0.0);
	vec2 tc3 = vec2(0.0);


	BilinearTexCoord(iTc, f, tc0, tc1, tc2, tc3);

	vec4 val = vec4(0.0);

#ifdef GLES_2	
	val.x = texture2D(tex, tc0).a;
	val.y = texture2D(tex, tc1).a;
	val.z = texture2D(tex, tc2).a;
	val.w = texture2D(tex, tc3).a;
#else	
	val.x = texture2D(tex, tc0).r;
	val.y = texture2D(tex, tc1).r;
	val.z = texture2D(tex, tc2).r;
	val.w = texture2D(tex, tc3).r;
#endif

	return val * 255.0;
}

//================================================================================

float MapRange(float fromMin, float fromMax, float toMin, float toMax, float s)
{
	return toMin + (s - fromMin) * (toMax - toMin) / (fromMax - fromMin);
}


float RoundToNearest(float number, float multiple)
{
	return ceil(number / multiple) * multiple;	
}

#ifdef GLOBE_MODE
//https://gamedev.stackexchange.com/questions/184388/fragment-shader-map-dot-texture-repeatedly-over-the-sphere/
vec2 CalcGlobeTextureRepeat(vec2 gps, float density)
{
    vec2 longLat = gps;
    longLat.x = MapRange(-PI, PI, -1.0, 1.0, gps.x);
    longLat.y = MapRange(-PI_2, PI_2, -0.5, 0.5, gps.y);
    
    // Count how many rows we can fit top to bottom at our selected density.
    float rowCount = ceil(density);
    // If it's odd, the 0th row sits at the equator. Otherwise, we need to shift it half a row.
    float even = mod(rowCount + 1.0, 2.0);

    // Scale and rount our latitude to find which row this fragment is in.
    float y = longLat.y * rowCount;
    float row = floor(y - 0.5 * even + 0.5);

    // Compute the radius around the sphere along the center of this row.
    float rowRadians = (row + 0.5 * even) * PI / rowCount;
    float rowRadius = cos(rowRadians);
       
    //not precise but without cos
    //rowRadius = sign(rowRadians) * MapRange(0.0, PI_2, 1.0, 0.0, abs(rowRadians));
    
    // Round this to get the number of texture repeats horizontally.
    float rowRepeats = floor(rowRadius * density * 2.0 + 0.5);

    // Make our texture coordinates.
    vec2 uv;
    uv.x = longLat.x * rowRepeats / 2.0;
    uv.y = y - row + (1.0 - even) * 0.5;
    
    return uv;
}

#endif

vec2 CalcUv()
{
#ifdef GLOBE_MODE
	return CalcGlobeTextureRepeat(gps, typeMapScale * 0.75);
#else
	return typeMapScale * vWorldPos.xy;
#endif
}

void main()
{		
	/*
	gl_FragColor = TEX2D(en_texModel, vTexCoord).rrrr;
	gl_FragColor.rgb = texId;
	gl_FragColor.a = 0.9;
	return;
	*/
	//================================================================
		
	vec4 stormColor = vec4(0.0);
	vec4 freezeColor = vec4(0.0);
	vec4 snowColor = vec4(0.0);
	vec4 rainColor = vec4(0.0);
	vec4 fogColor = vec4(0.0);	
	
	float unpackedTypeNon = PrepareNoInterpolationForTypes(vTexCoord, en_texModel);
		  
	//only one of these conditions can be valid for a single pixel
		
	if (IsStorm_NoInterpolation(unpackedTypeNon))	
	{		
		stormColor = texture2D(stormMap, CalcUv()).rgba;
		stormColor.a *= opacityRainStormFreezeSnow.y;
	}
	else if (IsFreezeRain_NoInterpolation(unpackedTypeNon))
	{
		freezeColor = texture2D(freezeMap, CalcUv()).rrrg;
		freezeColor.a *= opacityRainStormFreezeSnow.z;
	}	
	else if (IsFog_NoInterpolation(unpackedTypeNon))	
	{
		fogColor = texture2D(fogMap, CalcUv()).rrrg;
		fogColor.a *= opacityFog;
	}	
	else if (IsRain_NoInterpolation(unpackedTypeNon))
	{
		rainColor = texture2D(rainMap, CalcUv()).rrrg;
		rainColor.a *= opacityRainStormFreezeSnow.x;
	}
	else if (IsSnow_NoInterpolation(unpackedTypeNon))
	{
		snowColor = texture2D(snowMap, CalcUv()).rrrg;
		snowColor.a *= opacityRainStormFreezeSnow.w;
	}
	/*
	else
	{
		vec2 f = vec2(0.0);
		vec4 unpackedTypeLin = PrepareBilinearForTypes(vTexCoord, en_texModel, f);

		if (IsRain_NoInterpolation(unpackedTypeNon))
		{
			rainColor = texture2D(rainMap, CalcUv()).rrrg;
			rainColor.a *= opacity;
		}
		else if (IsSnow_NoInterpolation(unpackedTypeNon))
		{
			snowColor = texture2D(snowMap, CalcUv()).rrrg;
			snowColor.a *= opacity;
		}
	}
	*/

	vec4 color = vec4(1.0, 1.0, 1.0, 1.0);
		
	color = snowColor;
	color += freezeColor;
	color += stormColor;
	color += rainColor;
	color += fogColor;

	gl_FragColor = color;					
}
